#!/usr/bin/python3.5

import os
import sys
import signal
import unittest

from modules.scheduling.scheduler import AuraScheduler
from modules.communication.liquidsoap.communicator import LiquidSoapCommunicator
from modules.communication.redis.adapter import ServerRedisAdapter
from modules.web.routes import Routes
from libraries.base.logger import AuraLogger

from testing.test import TestConfig, TestLogger

class Aura(AuraLogger):
    server = None
    messenger = None
    controller = None

    # ------------------------------------------------------------------------------------------ #
    def __init__(self):
        super(Aura, self).__init__()

    def test_yourself(self):
        self.run_test(TestConfig)
        self.run_test(TestLogger)

    def run_test(self, testingClass):
        runner = unittest.TextTestRunner()
        result = runner.run(unittest.makeSuite(testingClass))

    def startup(self):

        server = object

        #        self.controller = AuraController(self.config)
        # create scheduler and ls_communicator
        self.liquidsoapcommunicator = LiquidSoapCommunicator(self.config)
        self.scheduler = AuraScheduler(self.config)

        # give both a reference of each other
        self.liquidsoapcommunicator.scheduler = self.scheduler
        self.scheduler.liquidsoapcommunicator = self.liquidsoapcommunicator

        # create the redis adapter
        self.messenger = ServerRedisAdapter()
        self.messenger.config = self.config
        self.messenger.scheduler = self.scheduler
        self.messenger.liquidsoapcommunicator = self.liquidsoapcommunicator

        def receive_signal(signum, stack):
            print("received signal")
            server.reload()

        signal.signal(signal.SIGUSR1, receive_signal)

        # wait for redis message
        self.join_comm()

        # start the web service
        self.start_web_service()

    def join_comm(self):
        # start listener thread
        self.messenger.start()

    def start_web_service(self):
        try:
            self.logger.info("Listening on Port 5000 for API or Webcalls")
            Routes(self.scheduler, self.liquidsoapcommunicator, self.messenger)
        except OSError as e:
            self.messenger.halt()
            self.logger.critical("AuraEngine already running? Exception: " + e.strerror + ". Exiting...")
            os._exit(0)


# # ## ## ## ## ## # #
# # ENTRY FUNCTION # #
# # ## ## ## ## ## # #
def main():
    aura = Aura()

    if len(sys.argv) >= 2 and sys.argv[1] == "--na-gspiast-di-?":
        aura.test_yourself()
    else:
        aura.startup()


# # ## ## ## ## ## ## # #
# # End ENTRY FUNCTION # #
# # ## ## ## ## ## ## # #


if __name__ == "__main__":
    main()
